#include <iostream>
#include <fstream>
#include <string>

#include <boost/lexical_cast.hpp>
#include <boost/tokenizer.hpp>

#include "IC.h"
#include "Globals.h"
#include "Utilities.h"
#include "AsmFileParser.h"
#include "Loader.h"

using namespace boost;
using namespace std;

string getString(string s) {
	string v;

	bool read = false;
	for (size_t i = 0;  i < s.size();  ++i) {
		if (s[i] == '\"') {
			read = !read;
		}
		else {
			if (read) v += s[i];
		}
	}
	
	return v;
}

vector<DataItem> parseDataSegment(ifstream& file) {
	typedef tokenizer<char_separator<char> > tokenizer;

	vector<DataItem> data;

	char line[100];
	file.getline(line, 80);

	if (!strcmp(line, DATA_SEGMENT)) {
		while (strcmp(line, CODE_SEGMENT)) {
			file.getline(line, 80);
			string item = line;
			
			char_separator<char> sep(" ");
			tokenizer tok(item, sep);
			for (tokenizer::iterator itr = tok.begin();  itr != tok.end();  ++itr) {
				string token = *itr;

				if (token == FLOAT_CONST) {
					float v = lexical_cast<float>(*(++itr));
					data.push_back(DataItem(v));
				}
				else if (token == BLOCK_DEF) {
					int v = lexical_cast<int>(*(++itr));
					data.push_back(DataItem(v));
				}
				else if (token == ASCIIZ_DEF) {
					string v = getString(item);	// could probably be done using the tokenizer
					data.push_back(DataItem(v));
					break;	// skip other tokens
				}
				else
					return data;		// data segment done
			}
		}
	}

	return data;
}

vector<Instruction> parseCodeSegment(ifstream& file) {
	typedef tokenizer<char_separator<char> > tokenizer;
	
	vector<Instruction> code;

	char line[MAX_ASM_LINE];

	while (strcmp(line, ASM_END)) {
		file.getline(line, MAX_ASM_LINE);
		string item = line;
		
		if (item != ASM_END && item != "") {
			char_separator<char> sep(" ,");
			tokenizer tok(item, sep);
			
			Instruction instr;
			int part = 0;
			for (tokenizer::iterator itr = tok.begin();  itr != tok.end();  ++itr, ++part) {
				string token = *itr;

				switch (part) {
					case 0: instr.mnemonic = token;  instr.opcode = getOpCode(token);  break;
					case 1: instr.optr1 = lexical_cast<int>(token);  break;
					case 2: instr.optr2 = lexical_cast<int>(token);  break;
				}
			}

			code.push_back(instr);
		}
	}

	return code;
}

void loadAsmFile(string asmFile, INT32*& dataPtr, INT32*& textPtr, ofstream* out) {
	ifstream file(asmFile.c_str(), ifstream::in);

	vector<DataItem> data = parseDataSegment(file);
	vector<Instruction> code = parseCodeSegment(file);

	loadData(data, dataPtr, out);
	loadCode(code, textPtr, out);
}
