#include <iostream>

#include "boost/lexical_cast.hpp"

#include "Globals.h"
#include "Utilities.h"
#include "Loader.h"

using namespace std;

//*******************************************************************
//*******************************************************************

string format(string w, int bytes[], bool byteFormat) {
	string s;
	
	if (byteFormat) {
		for (size_t i = 0;  i < w.size();  ++i) {
			s += w[i];

			if (i + 1 == 8 || i + 1 == 16 || i + 1 == 24)
				s += ":";
		}
	}
	else {
		for (size_t i = 0;  i < w.size();  ++i) {
			s += w[i];

			if (i + 1 == 12 || i + 1 == 24)
				s += ":";
		}
	}

	return s;
}

//*******************************************************************

string formatDecimal(int n) {
	string s = boost::lexical_cast<string>(n);
	for (size_t i = 3 - s.length();  i > 0;  --i) s = " " + s;
	
	return s;
}

//*******************************************************************

pair<string, string> showWord(INT32* addr, bool byteFormat) {
	int mask = 1;
	string binary;
	for (int i = 0;  i < 32;  ++i) {
		if (*addr & mask)
			binary = "1" + binary;
		else
			binary = "0" + binary;

		mask *= 2;
	}

	int bmask = 255;
	int bytes[4];
	bytes[0] = *addr & bmask;
	bytes[1] = *addr >> 8 & bmask;
	bytes[2] = *addr >> 16 & bmask;
	bytes[3] = *addr >> 24 & bmask;

	string bin = format(binary, 0, byteFormat);
	string dec = formatDecimal(bytes[3]) + ":" +
						formatDecimal(bytes[2]) + ":" +
						formatDecimal(bytes[1]) + ":" +
						formatDecimal(bytes[0]);
	
	return pair<string, string>(bin, dec);
}

//*******************************************************************

void showInstrMem(INT32* addr, Instruction instr, ofstream* out) {
	const string space = "     ";

	pair<string, string> word = showWord(addr, true);
	string s = boost::lexical_cast<string>(addr) + ": " + word.first + space + word.second;
	*out << "\n" << s << space << instr;
}

//*******************************************************************

int getOpCode(string instr) {
	int i = 0;
	while (mnemonic[i++] != "") {
		if (mnemonic[i] == instr) return i;
	}

	return -1;
}

//*******************************************************************

ostream& operator<<(ostream& os, Instruction instr) {
	os << instr.mnemonic;

	if (instr.optr1 > -1) os << " " << instr.optr1;
	if (instr.optr2 > -1) os << ", " << instr.optr2;

	return os;
}
